@@ -12,6 +12,7 @@ from pythautomata.model_comparators.wfa_partition_comparison_strategy import WFA
 from pythautomata.utilities.probability_partitioner import QuantizationProbabilityPartitioner, TopKProbabilityPartitioner
 
 from pymodelextractor.teachers.pdfa_teacher import PDFATeacher
+from pymodelextractor.teachers.pac_batch_probabilistic_teacher import PACBatchProbabilisticTeacher
 
 from pythautomata.utilities import pdfa_generator
 from pythautomata.utilities import abbadingo_one_dfa_generator
@@ -406,4 +407,15 @@ class TestBoundedPDFAQuantizantionNAryTreeLearner(unittest.TestCase):
         result = learner.learn(teacher)
         extracted_model = result.model
         self.assertTrue(result.info['last_token_weight_queries_count'] > 0)        
+        self.assertTrue(result.info['equivalence_queries_count'] > 0)
+
+    def test_arg_max_partitioner3(self):     
+        model = self.generate_PDFA_with_3_different_states_according_to_argmax()
+        partitioner = TopKProbabilityPartitioner(1)
+        comparator = WFAPartitionComparator(partitioner)
+        teacher = PACBatchProbabilisticTeacher(model, epsilon = 0.05, delta = 0.05, comparator=comparator)
+        learner = BoundedPDFAQuantizationNAryTreeLearner(partitioner,10, 2, generate_partial_hipothesis=True, pre_cache_queries_for_building_hipothesis=True)
+        result = learner.learn(teacher)
+        extracted_model = result.model
+        self.assertTrue(result.info['last_token_weight_queries_count'] > 0)        
         self.assertTrue(result.info['equivalence_queries_count'] > 0)
\ No newline at end of file
